#ifndef INTERPOLATOR_H
#define INTERPOLATOR_H

#include <cstring>
#include <stdexcept>

template<class T> struct pwconstant {          // piecewise constant interpolation policy
    static T interpolate(int n, const T table[][2], T x) {
        int i;  // we'll need the counter value AFTER the loop
        
        if (x < table[0][0]) throw std::out_of_range("interpolator");
        
        for (i = 1; i < n-1; ++i)         // linear serch is inefficient!
            if (x < table[i][0]) break;   // this is only a naive example
        
        if (i == n && x != table[n-1][0]) throw std::out_of_range("interpolator");
        
        return table[i-1][1];
    }
};

template<class T, class I> class interpolator {
    int points;
    T (*table)[2];
    
public:
    interpolator() : points(-1), table(NULL) { }
    interpolator(int n) : points(n), table(new T[n][2]) { 
        for (int i = 0; i < n+1; ++i)
            table[i][0] = table[i][1] = 0.0;
    }
    interpolator(int n, const T d[][2]) : points(n), table(new T[n][2]) {
        memcpy(table, d, (n*2)*sizeof(T));
    }
    interpolator(int n, const T x[], const T y[]) : points(n), table(new T[n][2]) {
        for (int i = 0; i < n+1; ++i) {
            table[i][0] = x[i];
            table[i][1] = y[i];
        }
    }
    interpolator(const interpolator& p);
    
    ~interpolator() { delete[] table; }
    
    interpolator& operator= (const interpolator& p);

    int rows() const { return points; }
    
    T x(int n) const;   // getter 
    T y(int n) const;   // getter 
    void setxy(int n, T x, T y);    // setter
    
    T operator() (T x) const {
        if (!table) throw std::domain_error("uninitialized interpolator");
        return I::interpolate(points, table, x);
    }
};

template<class T, class I> interpolator<T, I>::interpolator(const interpolator<T, I>& p) : points(p.points), table(NULL) {
    if (points >= 0) {
        table = new T[points][2];
        memcpy(table, p.table, (points*2)*sizeof(T));
    }
}

template<class T, class I> interpolator<T, I>& interpolator<T, I>::operator= (const interpolator<T, I>& p) {
    if (this == &p) return *this;
    points = p.points;
    delete[] table; table = NULL;
    if (points >= 0) {
        table = new T[points];
        memcpy(table, p.table, (points*2)*sizeof(T));
    }
    return *this;
}

template<class T, class I> inline T interpolator<T, I>::x(int n) const {  // getter 
    if (!table) throw std::domain_error("uninitialized interpolator");
    if (n < 0 || n >= points) throw std::out_of_range("interpolator");
    return table[n][0];
}

template<class T, class I> inline T interpolator<T, I>::y(int n) const {  // getter 
    if (!table) throw std::domain_error("uninitialized interpolator");
    if (n < 0 || n >= points) throw std::out_of_range("interpolator");
    return table[n][1];
}

template<class T, class I> inline void interpolator<T, I>::setxy(int n, T x, T y) {  // setter 
    if (!table) throw std::domain_error("uninitialized interpolator");
    if (n < 0 || n >= points) throw std::out_of_range("interpolator");
    table[n][0] = x;
    table[n][1] = y;
}

#endif